@@ -3,7 +3,7 @@ use strict;
 use warnings;
 use 5.010;
 
-our $VERSION = '0.20';
+our $VERSION = '0.21';
 
 our $PANDOC_VERSION;    # a string like '1.16'
 $PANDOC_VERSION ||= $ENV{PANDOC_VERSION};
@@ -92,7 +92,9 @@ foreach my $name ( keys %ELEMENTS ) {
         sub {
             croak "$name expects $numargs arguments, but given " . scalar @_
               if @_ != $numargs;
-            bless { t => $name, c => ( @_ == 1 ? $_[0] : [@_] ) }, $class;
+            my $self = bless { t => $name, c => ( @_ == 1 ? $_[0] : [@_] ) }, $class;
+            $self->set_content(@_);
+            $self;
         },
         '$' x $numargs
     );
@@ -131,11 +133,10 @@ sub Pandoc::Document::DefinitionPair::definitions { $_[0]->[1] }
 # additional functions
 
 sub attributes($) {
-    
-    my $e = Span(['',[],[]],[]); # to make use of AttributesRole
 
+    my $e = Span(['',[],[]],[]); # to make use of AttributesRole
     $e->keyvals(@_);
-    
+
     return $e->attr;
 }
 
@@ -205,6 +206,13 @@ sub pandoc_json($) {
     sub meta        { $_[0]->[0]->{unMeta} }
     sub content     { $_[0]->[1] }
     sub is_document { 1 }
+    sub metavalue {
+        my $meta = $_[0]->meta;
+        return { map { $_ => $meta->{$_}->metavalue } keys %$meta }
+    }
+    sub string {
+        join '', map { $_->string } @{$_[0]->content}
+    }
 }
 
 {
@@ -219,7 +227,7 @@ sub pandoc_json($) {
     use subs qw(walk query transform);    # Silence syntax warnings
 
     sub to_json {
-        JSON->new->utf8->convert_blessed->encode( $_[0] );
+        JSON->new->utf8->canonical->convert_blessed->encode( $_[0] );
     }
 
     sub TO_JSON {
@@ -254,7 +262,15 @@ sub pandoc_json($) {
     }
 
     sub name        { $_[0]->{t} }
-    sub content     { $_[0]->{c} }
+    sub content     {
+       my $e = shift;
+       $e->set_content(@_) if @_;
+       $e->{c}
+    }
+    sub set_content { # TODO: document this
+       my $e = shift;
+       $e->{c} = @_ == 1 ? $_[0] : [@_]
+    }
     sub is_document { 0 }
     sub is_block    { 0 }
     sub is_inline   { 0 }
@@ -271,7 +287,7 @@ sub pandoc_json($) {
         if ( $_[0]->name =~ /^(Str|Code|Math|MetaString)$/ ) {
             return $_[0]->content;
         }
-        elsif ( $_[0]->name =~ /^(LineBreak|SoftBreak|Space|MetaString)$/ ) {
+        elsif ( $_[0]->name =~ /^(LineBreak|SoftBreak|Space)$/ ) {
             return ' ';
         }
         join '', @{
@@ -321,31 +337,35 @@ sub pandoc_json($) {
     package Pandoc::Document::AttributesRole;
     use Hash::MultiValue;
     use Scalar::Util qw(reftype blessed);
-    
+    use Carp qw(croak);
+
     my $IDENTIFIER = qr{\p{L}(\p{L}|[0-9_:.-])*};
 
     sub id {
         $_[0]->attr->[0] = "$_[1]" if @_ > 1;
-        $_[0]->attr->[0] 
+        $_[0]->attr->[0]
     }
 
-    sub classes { 
+    sub classes {
         my $e = shift;
-        if (@_) {
-            $e->attr->[1] = [];
-            $e->add_keyval('class', $_) for @_;
-        }
-        $e->attr->[1] 
+        croak 'Method classes() is not a setter' if @_;
+        $e->attr->[1]
     }
-    
+
     sub class {
         my $e = shift;
-        $e->classes(@_) if @_;
+        if (@_) {
+            $e->attr->[1] = [
+                grep { $_ ne '' }
+                map { split qr/\s+/, $_ }
+                map { (ref $_ and reftype $_ eq 'ARRAY') ? @$_ : $_ }
+                @_
+            ];
+        }
         join ' ', @{$e->classes}
     }
-    
-    # TODO: document this method
-    sub add_keyval {
+
+    sub add_attribute {
         my ($e, $key, $value) = @_;
         if ($key eq 'id') {
             $e->id($value);
@@ -361,18 +381,17 @@ sub pandoc_json($) {
         my $e = shift;
         if (@_) {
             my $attrs = @_ == 1 ? shift : Hash::MultiValue->new(@_);
-            $e->attr->[2] = [];
-            if (blessed $attrs and $attrs->isa('Hash::MultiValue')) {
-                $attrs->each(sub { $e->add_keyval(@_) });
-            } else {
-                while (my ($key,$value) = each %$attrs) {
-                    $e->add_keyval($key, $value);
-                }
+            unless (blessed $attrs and $attrs->isa('Hash::MultiValue')) {
+                $attrs = Hash::MultiValue->new(%$attrs);
             }
+            $e->attr->[1] = [] if exists $attrs->{class};
+            $e->attr->[2] = [];
+            $attrs->each(sub { $e->add_attribute(@_) });
         }
-        Hash::MultiValue->new(
-            map { @$_ } @{$e->attr->[2]}
-        )
+        my @h;
+        push @h, id => $e->id if $e->id ne '';
+        push @h, class => $e->class if @{$e->classes};
+        Hash::MultiValue->new( @h, map { @$_ } @{$e->attr->[2]} );
     }
 
     # TODO: rename and/or extend to keyvals check
@@ -415,6 +434,7 @@ sub pandoc_json($) {
 {
 
     package Pandoc::Document::Meta;
+    use Scalar::Util 'reftype';
     our $VERSION = $PANDOC::Document::VERSION;
     our @ISA     = ('Pandoc::Document::Element');
     sub is_meta { 1 }
@@ -491,6 +511,10 @@ sub Pandoc::Document::Table::TO_JSON {
     return $ast;
 }
 
+sub Pandoc::Document::MetaBool::set_content {
+    $_[0]->{c} = $_[1] && $_[1] ne 'false' && $_[1] ne 'FALSE'
+}
+
 sub Pandoc::Document::MetaBool::TO_JSON {
     return {
         t => 'MetaBool',
@@ -500,6 +524,31 @@ sub Pandoc::Document::MetaBool::TO_JSON {
     };
 }
 
+sub Pandoc::Document::MetaBool::metavalue {
+    $_[0]->{c} ? 1 : 0
+}
+
+sub Pandoc::Document::MetaMap::metavalue {
+    my $map = $_[0]->{c};
+    return { map { $_ => $map->{$_}->metavalue } keys %$map }
+}
+
+sub Pandoc::Document::MetaInlines::metavalue {
+    join '', map { $_->string } @{$_[0]->{c}}
+}
+
+sub Pandoc::Document::MetaBlocks::metavalue {
+    join "\n", map { $_->string } @{$_[0]}->{c}
+}
+
+sub Pandoc::Document::MetaList::metavalue {
+    [ map { $_->metavalue } @{$_[0]->{c}} ]
+}
+
+sub Pandoc::Document::MetaString::metavalue {
+    $_[0]->{c}
+}
+
 sub Pandoc::Document::Cite::TO_JSON {
     my $ast = Pandoc::Document::Element::TO_JSON( $_[0] );
     for my $citation ( @{ $ast->{c}[0] } ) {
@@ -607,11 +656,8 @@ names) are recognized. You can always manually create an attributes structure:
 
     [ $id, [ @classes ], [ [ key => $value ], ... ] ]
 
-Elements with attributes (element accessor method C<attr>) also provide the
-accessor method C<id>, C<classes>, C<class>, and C<keyvals>. The C<keyvals>
-accessor returns an instance of L<Hash::MultiValue> (but key-value pairs cannot
-be modified through this interface). All attribute accessors can also be used
-as setters.
+See also L<attribute methods|/ATTRIBUTE METHODS> to get and set element
+attributes.
 
 =head3 citation { ... }
 
@@ -653,7 +699,7 @@ methods specific to each element.
 Return the element as JSON encoded string. The following are equivalent:
 
     $element->to_json;
-    JSON->new->utf8->convert_blessed->encode($element);
+    JSON->new->utf8->canonical->convert_blessed->encode($element);
 
 Note that the JSON format changed from Pandoc 1.15 to Pandoc 1.16 by introduction
 of attributes to L<Link|/Link> and L<Image|/Image> elements. Since Pandoc::Elements
@@ -704,6 +750,15 @@ Transform the element tree with L<Pandoc::Walker>
 
 Returns a concatenated string of element content, leaving out all formatting.
 
+=head3 ATTRIBUTE METHODS
+
+Elements with attributes (element accessor method C<attr>) also provide the
+getter methods C<id>, C<classes>, C<class>, C<keyvals>, and setter methods
+C<id>, C<class>, C<keyvals>, C<add_attribute>. Method C<keyvals> returns a copy
+of id, class, and attribute key-value pairs as L<Hash::MultiValue>.  If used as
+setter, all key-value pairs (and optionally id and class if given) are
+replaced.
+
 =head2 BLOCK ELEMENTS
 
 =head3 BlockQuote
@@ -928,7 +983,7 @@ Generic container of L<inlines|/INLINE ELEMENTS> (C<content>) with attributes
 
 Plain text, a string (C<content>).
 
-    Str $text
+    Str $content
 
 =head3 Strikeout
 
@@ -956,17 +1011,59 @@ Superscripted text, a list of L<inlines|/INLINE ELEMENTS> (C<content>).
 
 =head2 METADATA ELEMENTS
 
-=head3 MetaBlocks
+Metadata can be provided in YAML syntax or via command line option C<-M>.  All
+metadata elements return true for C<is_meta>.  Metadata elements can be
+converted to unblessed Perl array references, hash references, and scalars with
+method C<metavalue>.  On the document level, metadata (document method C<meta>)
+is a hash reference with values being metadata elements. Document method
+C<metavalue> returns a flattened copy of this hash.
+
+=head3 MetaString
+
+A plain text string metadata value (C<content>).
+
+    MetaString $content
+
+MetaString values can also be set via pandoc command line client:
+
+    pandoc -M key=$content
 
 =head3 MetaBool
 
+A Boolean metadata value (C<content>). The special values C<"false"> and
+C<"FALSE"> are recognized as false in addition to normal false values (C<0>,
+C<undef>, C<"">...).
+
+    MetaBool $content
+
+MetaBool values can also be set via pandoc command line client:
+
+    pandoc -M key=true
+    pandoc -M key=false
+
 =head3 MetaInlines
 
+Container for a list of L<inlines|/INLINE ELEMENTS> (C<content>) in metadata.
+
+    MetaInlines [ @inlines ]
+
+=head3 MetaBlocks
+
+Container for a list of L<blocks|/BLOCK ELEMENTS> (C<content>) in metadata.
+
+    MetaInlines [ @blocks ]
+
 =head3 MetaList
 
+A list of other L<metadata elements|/METADATA ELEMENTS> (C<content>).
+
+    MetaList [ @values ]
+
 =head3 MetaMap
 
-=head3 MetaString
+A map of keys to other metadata elements.
+
+    MetaMap { %map }
 
 =head2 DOCUMENT ELEMENT
 
@@ -977,6 +1074,12 @@ Root element, consisting of metadata hash (C<meta>) and document element array
 
     Document $meta, [ @blocks ]
 
+Document C<metavalue> returns a copy of the metadata hash with all L<metadata
+elements|/METADATA ELEMENTS> flattened to unblessed values:
+
+    $doc->metavalue   # equivalent to
+    { map { $_ => $doc->meta->{$_}->metavalue } keys %{$doc->meta} }
+
 =head2 TYPE KEYWORDS
 
 The following document elements are only as used as type keywords in other
